#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @namespace pyfortified_requests

import json
import time


class Metrics(object):
    '''
    A container class for metrics.
    Intended to be used by RequestsFortified to capture api requests metrics.
    '''
    _metrics_dict = {}

    def inc(self, name, delta=1):
        """
        Increment counter <name>
        :param name: name of counter
        :param delta: the amount to increment
        :return: None
        """
        value = self._metrics_dict.setdefault(name, 0)
        self._metrics_dict[name] = value + delta

    def set(self, name, value):
        """
        Set gauge <name>
        :param name: name of gauge
        :param value: the value to set
        :return: None
        """
        self._metrics_dict[name] = value

    def add_sample(self, name, value):
        """
        Add a <value> sample to metric <name>
        :param name: name of metric
        :param value: the sample value to add
        :return: None
        """
        # Samples for metric <name> are kept in a list, as (time, value) pairs.
        samples = self._metrics_dict.setdefault(name, list())
        samples.append((time.time(), value))

    def dict(self):
        return self._metrics_dict

    def json(self):
        return str(json.dumps(self._metrics_dict))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @namespace pyfortified_requests

import logging
import requests
from requests.adapters import (HTTPAdapter, DEFAULT_POOLSIZE)
from urllib3.util.retry import Retry
from pyfortified_requests.support import (REQUEST_RETRY_HTTP_STATUS_CODES)
from pyfortified_requests.errors import (get_exception_message)

log = logging.getLogger(__name__)


class RequestsSessionClient(object):
    POOL_SIZE = DEFAULT_POOLSIZE
    request_buffer = []

    __session = None

    def __init__(self, retry_tries=3, retry_backoff=0.1, retry_codes=None, session=None):

        if session is not None:
            assert isinstance(session, requests.Session)
            self.session = session
        else:
            self.session = requests.Session()

            if retry_codes is None:
                retry_codes = set(REQUEST_RETRY_HTTP_STATUS_CODES)

            adapter = HTTPAdapter(
                max_retries=Retry(
                    total=retry_tries,
                    backoff_factor=retry_backoff,
                    status_forcelist=retry_codes,
                )
            )

            self.session.mount('http://', adapter)
            self.session.mount('https://', adapter)

    @property
    def session(self):
        return self.__session

    @session.setter
    def session(self, value):
        self.__session = value

    def request(self, request_method, request_url, **kwargs):
        extra_session_request = {'method': request_method, 'url': request_url}
        extra_session_request.update(kwargs)
        log.debug("Session Request: Details", extra=extra_session_request)
        try:
            return self.session.request(method=request_method, url=request_url, **kwargs)
        except Exception as ex:
            log.warning(
                "Session Request: Failed: %s" % get_exception_message(ex),
                extra=extra_session_request,
            )
            raise



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @namespace pyfortified_requests

__title__ = 'pyfortified-requests'
__version__ = '0.3.2'
__version_info__ = tuple(__version__.split('.'))

__python_required_version__ = (3, 0)

from pyfortified_requests.support.requests_session_client import (RequestsSessionClient)

from .pyfortified_requests import (RequestsFortified)
from .pyfortified_requests_download import (RequestsFortifiedDownload)
from .pyfortified_requests_upload import (RequestsFortifiedUpload)
from .errors import RequestsFortifiedErrorCodes as HttpStatusCode



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @namespace pyfortified_requests

from pyhttpstatus_utils import HttpStatusCode

class RequestsFortifiedErrorCodes(HttpStatusCode):

    REQ_ERR_UNASSIGNED = -1

    REQ_OK = 0  # Success process

    #
    # 6xx Request Fortified Errors
    #

    REQ_ERR_MODULE = 600  # Module Error

    REQ_ERR_ARGUMENT = 601
    # Invalid or missing argument provided discovered in this module.

    REQ_ERR_REQUEST = 602
    # Call to Python requests package has failed.

    REQ_ERR_SOFTWARE = 603
    # Exit code that means an internal software error was detected.

    REQ_ERR_UNEXPECTED_VALUE = 604
    # Unexpected value eitherreturned or null.

    REQ_ERR_REQUEST_HTTP = 605
    # An HTTP error occurred.

    REQ_ERR_REQUEST_CONNECT = 606
    # A Connection error occurred.

    REQ_ERR_REQUEST_REDIRECTS = 607

    REQ_ERR_RETRY_EXHAUSTED = 608  # Retry Exhausted

    REQ_ERR_UNEXPECTED_CONTENT_TYPE_RETURNED = 609  # Unexpected content-type returned

    REQ_ERR_UPLOAD_DATA = 610  # Error during data upload

    REQ_ERR_AUTH_ERROR = 611  # Auth Error
    REQ_ERR_AUTH_JSON_ERROR = 612  # Auth JSON Error
    REQ_ERR_AUTH_RESP_ERROR = 613  # Auth Response Error
    REQ_ERR_JSON_DECODING_ERROR = 614  # JSON Decoding Error

    REQ_ERR_CONNECT = 615  # Connection Error originating from a python builtins.py exception

    REQ_ERR_UNEXPECTED = 699  # Unexpected Error



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @namespace pyfortified_requests

from pyfortified_requests.errors.error_codes import RequestsFortifiedErrorCodes
from pyfortified_requests.exceptions.base import (RequestsFortifiedBaseError)


class RequestsFortifiedError(RequestsFortifiedBaseError):
    pass


class RequestsFortifiedClientError(RequestsFortifiedBaseError):
    pass


class RequestsFortifiedServiceError(RequestsFortifiedBaseError):
    pass


class RequestsFortifiedModuleError(RequestsFortifiedBaseError):
    pass


class RequestsFortifiedClientGoneError(RequestsFortifiedModuleError):
    """Request Mv Integration: Value error"""

    def __init__(self, **kwargs):
        error_code = kwargs.pop('error_code', None) or \
                     RequestsFortifiedErrorCodes.GONE
        super(RequestsFortifiedClientGoneError, self).__init__(error_code=error_code, **kwargs)


class RequestsFortifiedValueError(RequestsFortifiedModuleError):
    """Request Mv Integration: Value error"""

    def __init__(self, **kwargs):
        error_code = kwargs.pop('error_code', None) or \
                     RequestsFortifiedErrorCodes.REQ_ERR_ARGUMENT
        super(RequestsFortifiedValueError, self).__init__(error_code=error_code, **kwargs)


class RequestsFortifiedAuthenticationError(RequestsFortifiedModuleError):
    """Request Mv Integration: Authentication error"""

    def __init__(self, **kwargs):
        error_code = kwargs.pop('error_code', None) or \
                     RequestsFortifiedErrorCodes.REQ_ERR_AUTH_ERROR
        super(RequestsFortifiedAuthenticationError, self).__init__(error_code=error_code, **kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @namespace pyfortified_requests

from .error_codes import (RequestsFortifiedErrorCodes)
from .error_desc import (
    error_desc,
    error_name,
)
from .errors_traceback import (
    get_exception_message,
    print_traceback,
    print_limited_traceback,
    print_traceback_stack,
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @namespace pyfortified_requests

from .base import RequestsFortifiedBaseError

from .custom import (
    RequestsFortifiedError,
    RequestsFortifiedClientError,
    RequestsFortifiedServiceError,
    RequestsFortifiedModuleError,
    RequestsFortifiedValueError,
    RequestsFortifiedAuthenticationError,
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @namespace pyfortified_requests

from pyhttpstatus_utils import HTTP_STATUS_DESC_DICT
from pyhttpstatus_utils import HTTP_STATUS_PHRASE_DICT

REQUESTS_FORTIFIED_ERROR_NAME_DICT = {
    -1: 'Unassigned',
    0: 'Success',
    600: 'Module Error',
    601: 'Argument Error',
    602: 'Request Error',
    603: 'Software Error',
    604: 'Unexpected Value',
    605: 'Request HTTP',
    606: 'Request Connect',
    607: 'Request Redirect',
    608: 'Retry Exhausted',
    609: 'Unexpected content-type returned',
    610: 'Upload Data Error',
    611: 'Auth Error',
    612: 'Auth JSON Error',
    613: 'Auth Response Error',
    614: 'JSON Decoding Error',
    699: 'Unexpected Error'
}

REQUESTS_FORTIFIED_ERROR_DESC_DICT = {
    -1: 'Unassiged exit condition',
    0: 'Successfully completed',
    600: 'Error occurred somewhere within module',
    601: 'Invalid or missing argument provided',
    602: 'Unexpected request failure',
    603: 'Unexpected software error was detected',
    604: 'Unexpected value returned',
    605: 'Request HTTP error occurred',
    606: 'Request Connection error occurred',
    607: 'Request Redirect',
    608: 'Retry Exhausted',
    609: 'Unexpected content-type returned',
    610: 'Upload Data Error',
    611: 'Auth Error',
    612: 'Auth JSON Error',
    613: 'Auth Response Error',
    614: 'JSON Decoding Error',
    699: 'Unexpected Error'
}


def error_name(error_code, return_bool=False):
    """Provide definition of Error Code

    Args:
        error_code:

    Returns:

    """
    if error_code is None or not isinstance(error_code, int):
        return "Error Code: Invalid Type: %d" % error_code

    exit_code_name_ = HTTP_STATUS_PHRASE_DICT.get(error_code, None)
    if exit_code_name_ is not None:
        return exit_code_name_

    exit_code_name_ = REQUESTS_FORTIFIED_ERROR_NAME_DICT.get(error_code, None)
    if exit_code_name_ is not None:
        return exit_code_name_

    return False if return_bool else "Error Code: Undefined: %d" % error_code


def error_desc(error_code, return_bool=False):
    """Provide definition of Error Code

    Args:
        error_code:

    Returns:

    """
    if error_code is None or not isinstance(error_code, int):
        return "Error Code: Invalid Type: %d" % error_code

    exit_code_description_ = HTTP_STATUS_DESC_DICT.get(error_code, None)
    if exit_code_description_ is not None:
        return exit_code_description_

    exit_code_description_ = REQUESTS_FORTIFIED_ERROR_DESC_DICT.get(error_code, None)
    if exit_code_description_ is not None:
        return exit_code_description_

    return False if return_bool else "Error Code: Undefined: %d" % error_code



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @namespace pyfortified_requests

import sys


#  Check Python Version
#
def python_check_version(required_version):
    """Check Python Version
    :param: required_version
    """
    assert isinstance(required_version, tuple)

    current_version = sys.version_info
    if current_version[0] == required_version[0] and \
       current_version[1] >= required_version[1]:
        pass
    elif current_version[0] > required_version[0]:
        pass
    else:
        sys.stderr.write(
            "[%s] - Error: Python interpreter must be %d.%d or greater"
            " to use this library, current version is %d.%d.\n" %
            (sys.argv[0], required_version[0], required_version[1], current_version[0], current_version[1])
        )
        sys.exit(-1)
    return 0


def bytes_to_human(size, precision=2):
    # http://code.activestate.com/recipes/578019
    # >>> bytes_to_human(10000)
    # '9.8K'
    # >>> bytes_to_human(100001221)
    # '95.4M'
    suffixes = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
    prefix = {}
    for i, suffix in enumerate(suffixes):
        prefix[suffix] = 1 << (i + 1) * 10
    for suffix in reversed(suffixes):
        if size >= prefix[suffix]:
            value = float(size) / prefix[suffix]
            return '%.*f %sB' % (precision, value, suffix)
    return "%s B" % size


def base_class_name(obj):
    return obj.__class__.__name__


def full_class_name(obj):
    try:
        return obj.__module__ + "." + obj.__class__.__name__
    except Exception as ex:
        return obj.__class__.__name__


def urlencode_dict(request_data):
    assert isinstance(request_data, dict)

    request_data_query = ""
    for key in sorted(request_data.keys()):
        request_data_query += str(key) + '=' + str(request_data[key]) + "&"

    return request_data_query[:-1]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @namespace pyfortified_requests

import ujson as json
import logging

import bs4
from requests_toolbelt.utils import dump
import xmltodict
from pprintpp import pprint
from pyhttpstatus_utils import (
    get_http_status_desc,
    get_http_status_type,
    is_http_status_successful,
)

from pyfortified_requests import (
    __python_required_version__,
)
from pyfortified_requests.errors import (
    get_exception_message,
    RequestsFortifiedErrorCodes,
)
from pyfortified_requests.exceptions import (
    RequestsFortifiedModuleError,
)
from pyfortified_requests.support.utils import (
    bytes_to_human,
    base_class_name,
    python_check_version,
)
from safe_cast import (
    safe_int,
    safe_str,
)
from .parse import (requests_response_text_html)

log = logging.getLogger(__name__)

python_check_version(__python_required_version__)


def validate_response(
    response,
    request_curl,
    request_label=None,
):
    """Validate response

    Args:
        response:
        request_curl:
        request_label:

    Returns:

    """
    response_extra = {}
    if request_label is None:
        request_label = 'Validate Response'

    if not response:
        error_message = "{0}: Failed: None".format(request_label)
        log.error(error_message, extra=response_extra)

        raise RequestsFortifiedModuleError(
            error_message=error_message,
            error_request_curl=request_curl,
            error_code=RequestsFortifiedErrorCodes.REQ_ERR_SOFTWARE
        )

    log.debug("{0}: Defined".format(request_label), extra=response_extra)

    response_extra.update({'http_status_code': response.status_code})

    # Not using hasattr on purpose
    # Assuming positive approach will give us speedup since when attribute exists
    # hasattr takes double the time.
    # Anyway we use text attribute here to logging purpose only
    try:
        response_extra.update({'response_text_length': len(response.text)})
    except AttributeError:
        pass

    if response.headers:
        if 'Content-Type' in response.headers:
            response_headers_content_type = \
                safe_str(response.headers['Content-Type'])
            response_extra.update({'Content-Type': response_headers_content_type})

        if 'Content-Length' in response.headers:
            response_headers_content_length = \
                safe_int(response.headers['Content-Length'])
            response_extra.update({'Content-Length': bytes_to_human(response_headers_content_length)})

        if 'Content-Encoding' in response.headers:
            response_content_encoding = \
                safe_str(response.headers['Content-Encoding'])
            response_extra.update({'Content-Encoding': response_content_encoding})

        if 'Transfer-Encoding' in response.headers:
            response_transfer_encoding = \
                safe_str(response.headers['Transfer-Encoding'])
            response_extra.update({'Transfer-Encoding': response_transfer_encoding})

    if not is_http_status_successful(http_status_code=response.status_code):
        error_message = "{0}: Failed".format(request_label)
        log.error(error_message, extra=response_extra)

        raise RequestsFortifiedModuleError(
            error_message=error_message,
            error_request_curl=request_curl,
            error_code=RequestsFortifiedErrorCodes.REQ_ERR_SOFTWARE
        )

    log.debug("{0}: Success".format(request_label), extra=response_extra)


def validate_json_response(
    response,
    request_curl,
    request_label=None,
    response_content_type_expected='application/json',
    raise_ex_if_not_json_response=True
):
    """Validate JSON response.

    Args:
        response:
        request_curl
        request_label:
        response_content_type_expected:
        raise_ex_if_not_json_response:

    Returns:

    """
    validate_response(response, request_curl, request_label)

    json_response = None
    response_extra = {}
    if request_label is None:
        request_label = 'Validate JSON Response'

    response_extra.update({'Content-Type': response_content_type_expected})

    try:
        response_content_type = response.headers.get('Content-Type', None)

        if response.headers.get('Content-Type', None) is not None:
            is_valid_response_content_type = \
                response_content_type == response_content_type_expected or \
                response_content_type.startswith(response_content_type_expected)

            if is_valid_response_content_type:
                json_response = requests_response_json(
                    response=response,
                    request_curl=request_curl,
                    request_label=request_label,
                )
            elif response_content_type.startswith('text/html'):
                try:
                    response_content_html_lines = \
                        requests_response_text_html(
                            response=response
                        )
                except Exception as ex:
                    raise RequestsFortifiedModuleError(
                        error_message=request_label,
                        errors=ex,
                        error_request_curl=request_curl,
                        error_code=RequestsFortifiedErrorCodes.REQ_ERR_UNEXPECTED_CONTENT_TYPE_RETURNED
                    )

                raise RequestsFortifiedModuleError(
                    error_message="Content-Type: Expected: '{0}', Actual: '{1}'".format(response_content_type_expected, response_content_type),
                    errors=response_content_html_lines,
                    error_request_curl=request_curl,
                    error_code=RequestsFortifiedErrorCodes.REQ_ERR_UNEXPECTED_CONTENT_TYPE_RETURNED
                )
            else:
                raise RequestsFortifiedModuleError(
                    error_message="Content-Type: Expected: '{0}', Actual: '{1}'".format(response_content_type_expected, response_content_type),
                    error_request_curl=request_curl,
                    error_code=RequestsFortifiedErrorCodes.REQ_ERR_UNEXPECTED_CONTENT_TYPE_RETURNED
                )

    except AttributeError:
        raise RequestsFortifiedModuleError(
            error_message="Content-Type: Undefined",
            error_request_curl=request_curl,
            error_code=RequestsFortifiedErrorCodes.REQ_ERR_UNEXPECTED_CONTENT_TYPE_RETURNED
        )

    response_extra.update({
        'http_status_code': response.status_code,
        'raise_ex_if_not_json_response': raise_ex_if_not_json_response
    })

    log.debug(
        "{0}: Success: JSON".format(request_label),
        extra=response_extra,
    )

    return json_response


def requests_response_json(
    response,
    request_curl,
    request_label=None,
    raise_ex_if_not_json_response=True,
):
    """Get JSON from response from requests

    Args:
        response:
        request_curl:
        request_label:
        raise_ex_if_not_json_response:

    Returns:

    """
    json_response = None
    response_extra = {}
    if request_label:
        response_extra.update({'request_label': request_label})

    try:
        json_response = response.json()
        response_details_source = 'json'
        response_content_length = len(json_response)

        response_extra.update({
            'response_details_source': response_details_source,
            'response_content_length': response_content_length
        })
    except ValueError as json_decode_ex:
        log.error("Validate JSON Response: Failed: JSONDecodeError", extra=response_extra)

        data = dump.dump_all(response)
        pprint(data.decode('utf-8'))

        pprint(response.text)

        handle_json_decode_error(
            response_decode_ex=json_decode_ex,
            response=response,
            response_extra=response_extra,
            request_label=request_label,
            request_curl=request_curl
        )

    except Exception as ex:
        log.error("Validate JSON Response: Failed: Exception", extra=response_extra)

        pprint(response.text)

        handle_json_decode_error(
            response_decode_ex=ex,
            response=response,
            response_extra=response_extra,
            request_label=request_label,
            request_curl=request_curl
        )

    if json_response is None:
        if raise_ex_if_not_json_response:
            log.error("Validate JSON Response: Failed: None", extra=response_extra)

            raise RequestsFortifiedModuleError(
                error_message="Validate JSON Response: Failed: None",
                error_request_curl=request_curl,
                error_code=RequestsFortifiedErrorCodes.REQ_ERR_SOFTWARE
            )
        else:
            log.warning("Validate JSON Response: None", extra=response_extra)
    else:
        log.debug("Validate JSON Response: Valid", extra=response_extra)

    return json_response


def build_response_error_details(request_label, request_url, response):
    """Build gather status of Requests' response.

    Args:
        request_label:
        request_url:
        response:

    Returns:

    """
    http_status_code = \
        response.status_code
    http_status_type = \
        get_http_status_type(http_status_code)
    http_status_desc = \
        get_http_status_desc(http_status_code)

    response_status = "%s: %s: %s" % (http_status_code, http_status_type, http_status_desc)

    response_error_details = {
        'request_url': request_url,
        'request_label': request_label,
        'response_status': response_status,
        'response_status_code': http_status_code,
        'response_status_type': http_status_type,
        'response_status_desc': http_status_desc
    }

    if response.headers:
        if 'Content-Type' in response.headers:
            response_headers_content_type = \
                safe_str(response.headers['Content-Type'])
            response_error_details.update({'Content-Type': response_headers_content_type})

        if 'Content-Length' in response.headers and \
                response.headers['Content-Length']:
            response_headers_content_length = \
                safe_int(response.headers['Content-Length'])
            response_error_details.update({'Content-Length': response_headers_content_length})

        if 'Transfer-Encoding' in response.headers and \
                response.headers['Transfer-Encoding']:
            response_headers_transfer_encoding = \
                safe_str(response.headers['Transfer-Encoding'])
            response_error_details.update({'Transfer-Encoding': response_headers_transfer_encoding})

        if 'Content-Encoding' in response.headers and \
                response.headers['Content-Encoding']:
            response_headers_content_encoding = \
                safe_str(response.headers['Content-Encoding'])
            response_error_details.update({'Content-Encoding': response_headers_content_encoding})

    if hasattr(response, "reason") and response.reason:
        response_error_details.update({'response_reason': response.reason})

    response_details = None
    response_details_source = None

    try:
        response_details = response.json()
        response_details_source = 'json'
    except Exception:
        if hasattr(response, 'text') and \
                response.text and \
                len(response.text) > 0:
            response_details = response.text
            response_details_source = 'text'

            if response_details.startswith('<html'):
                response_details_source = 'html'
                soup_html = bs4.BeautifulSoup(response_details, "html.parser")
                # kill all script and style elements
                for script in soup_html(["script", "style"]):
                    script.extract()  # rip it out
                text_html = soup_html.get_text()
                lines_html = [line for line in text_html.split('\n') if line.strip() != '']
                lines_html = [line.strip(' ') for line in lines_html]
                response_details = lines_html

            elif response_details.startswith('<?xml'):
                response_details_source = 'xml'
                response_details = json.dumps(xmltodict.parse(response_details))

    response_error_details.update({
        'response_details': response_details,
        'response_details_source': response_details_source
    })

    # pprint(response_error_details)

    return response_error_details


def handle_json_decode_error(
    response_decode_ex,
    response,
    response_extra=None,
    request_label=None,
    request_curl=None,
):
    """Handle JSON Decode Error

    Args:
        response_decode_ex:
        response:
        response_extra:
        request_label:
        request_curl:

    Returns:

    """
    if response_extra is None:
        response_extra = {}

    if request_label:
        response_extra.update({'request_label': request_label})

    if hasattr(response, 'text') and \
            response.text and \
            len(response.text) > 0:
        response_details = response.text
        response_details_source = 'text'
        response_content_length = len(response_details)

        if response_details.startswith('<html'):
            response_details_source = 'html'
            soup_html = bs4.BeautifulSoup(response_details, "html.parser")
            # kill all script and style elements
            for script in soup_html(["script", "style"]):
                script.extract()  # rip it out
            text_html = soup_html.get_text()
            lines_html = [line for line in text_html.split('\n') if line.strip() != '']
            lines_html = [line.strip(' ') for line in lines_html]
            response_details = lines_html

        elif response_details.startswith('<?xml'):
            response_details_source = 'xml'
            response_details = json.dumps(xmltodict.parse(response_details))
        else:
            pprint(response_details)

        response_extra.update({
            'response_details': response_details,
            'response_details_source': response_details_source,
            'response_content_length': response_content_length,
            'error_exception': base_class_name(response_decode_ex),
            'error_details': get_exception_message(response_decode_ex)
        })

    log.error("Validate JSON Response: Failed: Invalid", extra=response_extra)

    raise RequestsFortifiedModuleError(
        error_message="Validate JSON Response: Failed: Invalid",
        errors=response_decode_ex,
        error_request_curl=request_curl,
        error_code=RequestsFortifiedErrorCodes.REQ_ERR_SOFTWARE
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#  @namespace pyfortified-requests
#

from __future__ import with_statement

# To install this library, open a Terminal shell,
# then run this file by typing:
#
# python setup.py install
#

import sys
import re
import codecs

from setuptools import setup

REQUIREMENTS = [
    req for req in open('requirements.txt')
    .read().split('\n')
    if req != ''
]

PACKAGES = [
    'pyfortified_requests',
    'pyfortified_requests.errors',
    'pyfortified_requests.exceptions',
    'pyfortified_requests.support',
    'pyfortified_requests.support.response'
]

with open('pyfortified_requests/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

if len(sys.argv) < 2 or sys.argv[1] == 'version':
    print(version)
    sys.exit()

CLASSIFIERS = [
    # How mature is this project? Common values are
    #   3 - Alpha
    #   4 - Beta
    #   5 - Production/Stable
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Natural Language :: English',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Natural Language :: English',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Topic :: Software Development :: Libraries :: Python Modules'
]

with codecs.open('README.rst', 'r', 'utf-8') as f:
    readme = f.read()

setup(
    name='pyfortified-requests',
    version=version,
    description="Extension to Python `requests` functionality.",
    long_description=readme,
    url='https://github.com/jeff00seattle/pyfortified-requests',
    download_url='https://github.com/jeff00seattle/pyfortified-requests/archive/v{0}.tar.gz'.format(version),
    keywords="pyfortified requests",
    license='MIT License',
    zip_safe=False,
    include_package_data=True,
    install_requires=REQUIREMENTS,
    packages=PACKAGES,
    package_data={'': ['LICENSE']},
    package_dir={'pyfortified-requests': 'pyfortified-requests'},
    classifiers=CLASSIFIERS
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @namespace pyfortified_requests


def csv_skip_last_row(iterator):
    """Skip last CSV row.

    Args:
        iterator:

    Returns:

    """
    prev = next(iterator)
    for item in iterator:
        yield prev
        prev = item



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @namespace pyfortified_requests

import sys
import requests
from pyfortified_requests import (__version__, __title__)
from pyhttpstatus_utils import (HttpStatusCode)

__all__ = [HttpStatusCode]

__MODULE_VERSION__ = __version__
__MODULE_VERSION_INFO__ = tuple(__MODULE_VERSION__.split('.'))
__MODULE_SIG__ = "%s/%s" % (__title__, __MODULE_VERSION__)

__TIMEZONE_NAME_DEFAULT__ = "UTC"

__PYTHON_VERSION__ = '%d.%d.%d' % (sys.version_info[0], sys.version_info[1], sys.version_info[2])
__PYTHON_SIG__ = "Python/%s" % __PYTHON_VERSION__

__USER_AGENT__ = "(%s, %s)" % (__MODULE_SIG__, __PYTHON_SIG__)

__LOGGER_NAME__ = __name__.split('.')[0]

HEADER_CONTENT_TYPE_APP_JSON = {'Content-Type': 'application/json'}
HEADER_CONTENT_TYPE_APP_URLENCODED = {'Content-Type': 'application/x-www-form-urlencoded'}

HEADER_USER_AGENT = \
    {'User-Agent': __USER_AGENT__}

REQUEST_RETRY_EXCPS = (
    requests.exceptions.ConnectTimeout, requests.exceptions.ReadTimeout, requests.exceptions.Timeout,
)

REQUEST_RETRY_HTTP_STATUS_CODES = [
    HttpStatusCode.INTERNAL_SERVER_ERROR,
    HttpStatusCode.BAD_GATEWAY,
    HttpStatusCode.SERVICE_UNAVAILABLE,
    HttpStatusCode.GATEWAY_TIMEOUT,
    HttpStatusCode.TOO_MANY_REQUESTS,
]

IRONIO_PARTITION = '/mnt/task'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @namespace pyfortified_requests

import os
import psutil
from pyfortified_requests.support.utils import bytes_to_human
from pyfortified_requests.support.constants import IRONIO_PARTITION


def mem_usage():
    virt = psutil.virtual_memory()
    return {
        'Mem': {
            'total': bytes_to_human(virt.total),
            'used': bytes_to_human(virt.used),
            'free': bytes_to_human(virt.free / 1024),
            'shared': bytes_to_human(getattr(virt, 'shared', 0)),
            'buffers': bytes_to_human(getattr(virt, 'buffers', 0)),
            'cached': bytes_to_human(getattr(virt, 'cached', 0))
        },
    }


def disk_usage(dir=None):
    if dir is None:
        dir = IRONIO_PARTITION

    if not os.path.exists(dir):
        dir = '/'

    usage = psutil.disk_usage(dir)
    return {
        'Disk:': {
            'path': dir,
            'total': bytes_to_human(usage.total),
            'used': bytes_to_human(usage.used),
            'free': bytes_to_human(usage.free),
            'percent': int(usage.percent)
        }
    }


def env_usage(dir=None):
    usage = {}
    usage.update(disk_usage(dir))
    usage.update(mem_usage())
    return usage
